#! /usr/bin/env python3

from PyQt4.QtCore import *
from PyQt4.QtGui import *

import os
mapFileName = os.path.dirname(os.path.realpath(__file__)) + "/world.jpeg"
mapMinLat, mapMaxLat =  -90,  90
mapMinLon, mapMaxLon = -180, 180
mapLatSpan = mapMaxLat - mapMinLat
mapLonSpan = mapMaxLon - mapMinLon

class EarthLocation:
	__slots__ = ["lat", "lon", "alt"]
	def __init__(self, lat, lon, alt = 0):
		self._checkLatLon(lat, lon)
		if lon == -180: lon = +180
		self.lat = lat ; self.lon = lon ; self.alt = alt
	def setLat(self, lat):
		self.lat = lat
	def setLon(self, lon):
		self.lon = lon
	def setAlt(self, alt):
		self.alt = alt
	def copy(self):
		return EarthLocation(self.lat, self.lon, self.alt)
	def __eq__(self, other):
		return self.lat == other.lat and self.lon == other.lon and self.alt == other.alt
	def _checkLatLon(self, lat, lon):
		if not (mapMinLat <= lat <= mapMaxLat and mapMinLon <= lon <= mapMaxLon):
			raise ValueError("Invalid latitude and/or longitude")

defaultLocation = EarthLocation(23.45, 82.5) # northern tropic, indian time zone reference meridian

def earthLocationForWidgetPos(pos, ppd):
	return EarthLocation(mapMaxLat - pos.y() / ppd, mapMinLon + pos.x() / ppd)

def widgetPosForEarthLocation(loc, ppd):
	return QPoint((loc.lon - mapMinLon) * ppd, (mapMaxLat - loc.lat) * ppd)

class MapWidget(QWidget):
	
	locationChanged = pyqtSignal(EarthLocation)
	
	def __init__(self):
		
		QWidget.__init__(self)
		
		self.pixmap = self.currentLocation = None
		self.tempPixmap = QPixmap(mapFileName)
		pixPerDegX = self.tempPixmap.width() / mapLonSpan
		pixPerDegY = self.tempPixmap.height() / mapLatSpan
		if pixPerDegX != pixPerDegY:
			self.setToolTip("ERROR: Provided image does not have the same aspect ratio as provided latitude/longitude extents")
			return
		
		self.pixPerDeg = pixPerDegX
		self.pixmap = self.tempPixmap
		self.setCurrentLocation(defaultLocation)
		self.resize(self.pixmap.size())
	
	def paintEvent(self, event):
		if self.pixmap is None: return
		painter = QPainter(self)
		painter.setRenderHint(QPainter.Antialiasing)
		painter.drawPixmap(0, 0, self.pixmap)
		oxygenBrickRed4 = QColor(226, 8, 0)
		redPen = QPen(oxygenBrickRed4)
		redPen.setWidth(2)
		painter.setPen(redPen)
		curPoint = widgetPosForEarthLocation(self.currentLocation, self.pixPerDeg)
		painter.drawLine(curPoint - QPoint(5,0), curPoint + QPoint(5,0))
		painter.drawLine(curPoint - QPoint(0,5), curPoint + QPoint(0,5))
		painter.drawEllipse(curPoint, 5, 5)
		painter.end()
	
	def setCurrentLocation(self, loc):
		self.currentLocation = loc
		self.update()
	
	def mousePressEvent(self, event):
		self.setCurrentLocation(earthLocationForWidgetPos(event.pos(), self.pixPerDeg))
		self.locationChanged.emit(self.currentLocation)

class MapScrollArea(QScrollArea):
	
	def __init__(self):
		QScrollArea.__init__(self)
		self.mapWidget = MapWidget()
		self.setWidget(self.mapWidget)
	
	def resizeEvent(self, event):
		self.centerOnCurrentLocation()
	
	def centerOnCurrentLocation(self):
		pos = widgetPosForEarthLocation(self.mapWidget.currentLocation, self.mapWidget.pixPerDeg)
		size = self.viewport().size()
		self.ensureVisible(pos.x(), pos.y(), size.width() / 2, size.height() / 2)

class MainWindow(QWidget):
	
	def __init__(self):
		
		QWidget.__init__(self)
		self.setWindowTitle("Map Test")
		
		w = self.mapScrollArea = MapScrollArea()
		
		w = self.mapWidget = self.mapScrollArea.mapWidget
		w.locationChanged.connect(self.mapLocationChanged)
		
		self.latLabel = QLabel("&Latitude")
		
		w = self.latSpinBox = QDoubleSpinBox()
		w.setRange(mapMinLat, mapMaxLat)
		w.setDecimals(2)
		self.latLabel.setBuddy(w)
		
		self.lonLabel = QLabel("L&ongitude")
		
		w = self.lonSpinBox = QDoubleSpinBox()
		w.setRange(mapMinLon, mapMaxLon)
		w.setDecimals(2)
		self.lonLabel.setBuddy(w)
		
		grid = [
		         [ self.latLabel, self.latSpinBox ],
		         [ self.lonLabel, self.lonSpinBox ]
		       ]
		l = self.gridLayout = QGridLayout()
		for row in range(len(grid)):
			for col in range(2):
				l.addWidget(grid[row][col], row, col)
		
		l = QHBoxLayout()
		l.addWidget(self.mapScrollArea)
		l.addLayout(self.gridLayout)
		self.setLayout(l)
		
		self.latSpinBox.setValue(defaultLocation.lat)
		self.lonSpinBox.setValue(defaultLocation.lon)
		
		self.setLatLonConnectionsEnabled(True)
		self.resize(640, 400)
	
	def setLatLonConnectionsEnabled(self, connect):
		for widget in self.latSpinBox, self.lonSpinBox:
			signal = widget.valueChanged[float]
			if connect: signal.connect(self.updateMap)
			else: signal.disconnect(self.updateMap)

	def updateMap(self):
		self.mapWidget.setCurrentLocation(EarthLocation(self.latSpinBox.value(), self.lonSpinBox.value()))
		self.mapScrollArea.centerOnCurrentLocation()
	
	def mapLocationChanged(self, loc):
		self.setLatLonConnectionsEnabled(False) # to avoid unnecessary redraw
		self.latSpinBox.setValue(loc.lat)
		self.lonSpinBox.setValue(loc.lon)
		self.setLatLonConnectionsEnabled(True) # restoring above disconnect

app = QApplication([])
mainWindow = MainWindow()
mainWindow.show()
app.exec_()
